#include <cmath>
#include <boost/thread.hpp>

#include <Wt/WApplication>
#include <Wt/WContainerWidget>
#include <Wt/WGridLayout>
#include <Wt/WGoogleMap>
#include <Wt/WTabWidget>

class TwoGoogleMapsApplication : public Wt::WApplication
{
  public:

    TwoGoogleMapsApplication (Wt::WEnvironment const & wEnvironment)
      : Wt::WApplication (wEnvironment)
    {
      setTitle ("two-google-maps example");
      setCssTheme ("polished");
      enableUpdates (true);

      class WContainerWidget : public Wt::WContainerWidget
      {
        public:

          WContainerWidget (Wt::WWidget * const wWidget)
          {
            Wt::WGridLayout * const layout = new Wt::WGridLayout ();

            layout->addWidget (wWidget, 0, 0, 1, 1);
            setLayout (layout);
            setMinimumSize (500, 400);
            setMaximumSize (500, 400);
          }
      };

      Wt::WTabWidget * const wTabWidget = new Wt::WTabWidget ();

      {
        Wt::WGoogleMap * const wGoogleMap = new Wt::WGoogleMap (Wt::WGoogleMap::Version3);
        boost::thread paintMarkerThread (boost::bind (&TwoGoogleMapsApplication::paintMarker, this, wGoogleMap));

        wTabWidget->addTab (wGoogleMap, "first tab");
      }
      wTabWidget->addTab (new Wt::WGoogleMap (Wt::WGoogleMap::Version3), "second tab");
      root ()->addWidget (new WContainerWidget (wTabWidget));
    }

  private:

    void paintMarker (Wt::WGoogleMap * const wGoogleMap)
    {
      Wt::WGoogleMap::Coordinate center (47.01887777, 8.651888);

      wGoogleMap->setCenter (center, 9);

      double angle = 0.0;

      for (;;) {
        boost::this_thread::sleep (boost::posix_time::milliseconds (1000));

        Wt::WApplication::UpdateLock updateLock (this);

        if (updateLock) {
          wGoogleMap->addMarker (Wt::WGoogleMap::Coordinate ((center.latitude () + (std::sin (angle * M_PI / 180.0) * .1)), (center.longitude () + (std::cos (angle * M_PI / 180.0) * .1))));
          triggerUpdate ();
        }
        angle += 10.0;
      }
    }
};

Wt::WApplication * createApplication (Wt::WEnvironment const & wEnvironment)
{
  return new TwoGoogleMapsApplication (wEnvironment);
}

int main (int argc, char ** argv)
{
  return WRun (argc, argv, &createApplication);
}

